/*
 * Uebungen zur Verifikationsnumerik
 * Wissenschaftliches Rechnen / Softwaretechnologie
 * Bergische Universitaet Wuppertal
 *
 * Implementierung von Ausschlussbereichen
 * bei der Nullstellensuche
 */

#include <cinterval.hpp>
#include <rmatrix.hpp>
#include <iostream>
#include <iomanip>
#include <string>

using namespace std;
using namespace cxsc;

// f(z) = z^4 - 2z^3 + z^2
cinterval f1(cinterval z)
{
	return ((z-2)*z+1)*z*z;
}

// f(z) = z^4 - (5+3i)z^3 + (6+7i)z^2 - (54-22i)z + 120 + 90i
// Nullstellen +-sqrt(2)(1+2i), 5, 3i
cinterval f2(cinterval z)
{
	cinterval p[5], res;
	p[4] = 1.0;
	p[3] = complex(-5,-3);
	p[2] = complex(6,7);
	p[1] = complex(-54,22);
	p[0] = complex(120,90);

	res = p[4];
	for(int i=3; i>=0; i--)
		res = (res*z)+p[i];

	return res;
}

// f(z) = e^z - 1
cinterval f3(cinterval z)
{
	return exp(z) - 1;
}

// f(z) = 0
cinterval f4(cinterval z)
{
	return cinterval(0.0);
}

// f(z) = 1
cinterval f5(cinterval z)
{
	return cinterval(1.0);
}

void plot(const rmatrix& graph, const cinterval& g, int nx, int ny) {
	// --------------------
	// bitte implementieren
	// --------------------
}

int main()
{
	bool easyInput = false;

	cinterval g;
	int nx(40), ny(18), fwahl(1);
	cinterval (*f)(cinterval); // Funktionszeiger
	string fstring;

	cout << "\nNullstellenausschlussbereich einer komplexen Funktion" << endl;
	cout << "=====================================================" << endl;
	cout << "Bitte Funktion waehlen: " << endl;
	cout << " (1) f(z) = z^4 - 2z^3 + z^2" << endl;
	cout << " (2) f(z) = z^4 - (5+3i)z^3 + (6+7i)z^2 - (54-22i)z + 120 + 90i" << endl;
	cout << " (3) f(z) = e^z - 1" << endl;
	cout << " (4) f(z) = 0" << endl;
	cout << " (5) f(z) = 1" << endl;
	cin >> fwahl;

	switch(fwahl) {
		case 1:
			f = f1;
			fstring = "f(z) = z^4 - 2z^3 + z^2";
			if(easyInput) {
				"([-8,8],[-8,8])" >> g;
				nx = 80;
				ny = 40;
			}
			break;
		case 2:
			f = f2;
			fstring = "f(z) = z^4 - (5+3i)z^3 + (6+7i)z^2 - (54-22i)z + 120 + 90i";
			if(easyInput) {
				"([-8,8],[-8,8])" >> g;
				nx = 80;
				ny = 40;
			}
			break;
		case 3:
			f = f3;
			fstring = "f(z) = e^z - 1";
			if(easyInput) {
				"([-8,8],[-8,8])" >> g;
				nx = 80;
				ny = 40;
			}
			break;
		case 4:
			f = f4;
			fstring = "f(z) = 0";
			if(easyInput) {
				"([-100,100],[-100,100])" >> g;
				nx = 80;
				ny = 40;
			}
			break;
		case 5:
		default:
			f = f5;
			fstring = "f(z) = 1";
			if(easyInput) {
				"([-100,100],[-100,100])" >> g;
				nx = 80;
				ny = 40;
			}
			break;
	}

	if(!easyInput) {
		cout << "Bitte das zu untersuchende Gebiet G (als komplexes Rechteckintervall) " << endl;
		cout << "eingeben: ";
		cin >> g;

		cout << "Bitte die Aufloesung eingeben (z.B. 70 25  oder  40 20): ";
		cin >> nx >> ny;
	}
	cout << endl;
	cout << "Funktion: " << fstring << endl;
	cout << "Gebiet: " << g << endl;
	cout << "Aufloesung: " << nx << " x " << ny << endl;
	cout << endl;

	rmatrix graph(nx,ny);
	graph = 0;

	// ---------------------------------------
	// Hier bitte Ausschlussbereiche berechnen
	// ---------------------------------------

	// Ausgabe
	plot(graph, g, nx, ny);

	return 0;
}
